/*
   Create a very simple hierarchila scene of
   three squares "hinged" to each other at two
   corners. There are three angles that can be
   set in this scene, one angle at each lower
   left-hand corner of a square.

                             +---+
                             |   |
                       +-----+---+
                       |     |
                       |     |
            +----------+-----+
            |          |
            |          |
            |          |
            |          |
            +----------+
*/

import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.awt.GLJPanel;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.glu.GLU;
import com.jogamp.opengl.util.FPSAnimator;

import javax.swing.JFrame;

public class Squares implements GLEventListener {

   private int frameNumber = 0;

   public Squares()
   {
      // Get a capabilities object from a GL2 profile.
      final GLProfile glProfile = GLProfile.get(GLProfile.GL2);
      final GLCapabilities glCapabilities = new GLCapabilities(glProfile);

      // Use it to create a GLJPanel.
      final GLJPanel glDrawable = new GLJPanel(glCapabilities);
      glDrawable.setPreferredSize(new java.awt.Dimension(400, 400));

      // Connect the GLJPanel with our GLEventListener implementation.
      glDrawable.addGLEventListener( this );

      // Add the GLJPanel to a JFrame.
      final JFrame jFrame = new JFrame ("Squares");
      jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      jFrame.getContentPane().add(glDrawable);
      jFrame.pack();
      jFrame.setLocationRelativeTo(null);
      jFrame.setVisible(true);

      // Create an animation thread.
      final FPSAnimator animator = new FPSAnimator(glDrawable, 30, true);
      animator.start();
   }


   @Override
   public void init(GLAutoDrawable drawable)
   {
    //drawable.setGL(new com.jogamp.opengl.TraceGL2((GL2)drawable.getGL(), System.err));

      final GL2 gl = drawable.getGL().getGL2();
      final GLU glu = new GLU();  // get GL Utilities

      // Initialize OpenGL state
      // attributes
      gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f); // white background

      // set up viewing
      gl.glMatrixMode(GL2.GL_PROJECTION);
      gl.glLoadIdentity();
      gl.glOrtho(-2.0f, 2.0f, -2.0f, 2.0f, -1.0f, 20f); // orthographic viewing volume
      //glu.gluPerspective(90, 1.0, 0.1, 20); // perspective viewing volume
      gl.glMatrixMode(GL2.GL_MODELVIEW);
   }

   @Override
   public void display(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glClear(GL2.GL_COLOR_BUFFER_BIT); // clear the graphics panel

      gl.glLoadIdentity();
      // Draw an xy-axis.
      gl.glLoadIdentity();
      gl.glColor3d(0.0, 0.0, 0.0);
      gl.glBegin(GL2.GL_LINES);
        gl.glVertex3d(-1.8,  0.0, 0.0);
        gl.glVertex3d( 1.8,  0.0, 0.0);
        gl.glVertex3d( 0.0, -1.8, 0.0);
        gl.glVertex3d( 0.0,  1.8, 0.0);
      gl.glEnd();

      drawSquares(frameNumber, 2.0*frameNumber, 4.0*frameNumber, drawable);
      gl.glFlush();

      frameNumber = (frameNumber + 1); // one degre per frame
   }

   private static void drawSquares(final double angle1,
                                   final double angle2,
                                   final double angle3,
                                   final GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();
      gl.glRotated(angle1, 0,0,1);     // new coord system
      drawSquare(drawable);
      gl.glTranslated(1.0, 1.0, 0.0);  // new coord system
      gl.glRotated(angle2, 0,0,1);     // new coord system
      gl.glScaled(0.5, 0.5, 1.0);      // new coord system
      drawSquare(drawable);
      gl.glTranslated(1.0, 1.0, 0.0);  // new coord system
      gl.glRotated(angle3, 0,0,1);     // new coord system
      gl.glScaled(0.5, 0.5, 1.0);      // new coord system
      drawSquare(drawable);
   }

   private static void drawSquare(final GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();
      gl.glBegin(GL2.GL_TRIANGLES);
         gl.glColor3f( 1.0f, 0.0f, 0.0f);
         gl.glVertex3f(0.0f, 0.0f, 0.0f);
         gl.glColor3f( 0.0f, 1.0f, 0.0f);
         gl.glVertex3f(0.0f, 1.0f, 0.0f);
         gl.glColor3f( 1.0f, 1.0f, 0.0f);
         gl.glVertex3f(1.0f, 0.0f, 0.0f);

         gl.glColor3f( 1.0f, 1.0f, 0.0f);
         gl.glVertex3f(1.0f, 0.0f, 0.0f);
         gl.glColor3f( 0.0f, 1.0f, 0.0f);
         gl.glVertex3f(0.0f, 1.0f, 0.0f);
         gl.glColor3f( 0.0f, 0.0f, 1.0f);
         gl.glVertex3f(1.0f, 1.0f, 0.0f);
      gl.glEnd();
   }


   @Override
   public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h)
   {
   }

   @Override
   public void dispose(GLAutoDrawable drawable)
   {
   }

   public static void main(String[] args)
   {
      javax.swing.SwingUtilities.invokeLater(
         () -> new Squares()
      );
   }
}
