/*
   Create a very simple hierarchila scene of
   five boxes stacked on top of each other.
   Moving a lower box left or right should move
   the boxes above it.

             +---+                 +---+
             |   |                 |   |
          +--+---+----+      +-----+---+-+
          |           |      |           |
          |           |      |           |
      +---+-----------+------+-----------+--+
      |                                     |
      |                                     |
      |                                     |
      +-------------------------------------+
*/

import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.awt.GLJPanel;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.glu.GLU;
import com.jogamp.opengl.util.FPSAnimator;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

public class StackedBoxes2 implements GLEventListener
{
   private final double width1  = 3.0;
   private final double height1 = 0.8;
   private final double width2  = 2.0;
   private final double height2 = 0.6;
   private final double width3  = 1.0;
   private final double height3 = 0.4;
   private final double width4  = 2.0;
   private final double height4 = 0.6;
   private final double width5  = 1.0;
   private final double height5 = 0.4;

   public static double d1 = 0.0;
   public static double d2 = 0.0;
   public static double d3 = 0.0;
   public static double d4 = 1.0;
   public static double d5 = 0.0;

   public StackedBoxes2()
   {
      // Get a capabilities object from a GL2 profile.
      final GLProfile glProfile = GLProfile.get(GLProfile.GL2);
      final GLCapabilities glCapabilities = new GLCapabilities(glProfile);

      // Use it to create a GLJPanel.
      final GLJPanel glDrawable = new GLJPanel(glCapabilities);
      glDrawable.setPreferredSize(new java.awt.Dimension(700, 200));

      // Connect the GLJPanel with our GLEventListener implementation.
      glDrawable.addGLEventListener( this );

      // Add the GLJPanel to a JFrame.
      final JFrame jFrame = new JFrame ("Stacked Boxes");
      jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

      final JSlider js1  = new JSlider(0, 100, 0);
      final JSlider js2  = new JSlider(0, 100, 0);
      final JSlider js3  = new JSlider(0, 100, 0);
      final JSlider js4  = new JSlider(0, 100, 20);
      final JSlider js5  = new JSlider(0, 100, 0);
      JPanel sliderPanel = new JPanel(new GridLayout(5,1));
      sliderPanel.add(js5);
      sliderPanel.add(js4);
      sliderPanel.add(js1);
      sliderPanel.add(js2);
      sliderPanel.add(js3);


      jFrame.getContentPane().setLayout(new BorderLayout());
      jFrame.getContentPane().add(glDrawable, BorderLayout.CENTER);
      jFrame.getContentPane().add(sliderPanel, BorderLayout.SOUTH);

      jFrame.pack();
      jFrame.setLocationRelativeTo(null);
      jFrame.setVisible(true);

      js5.addChangeListener(new SliderChangeListener5());
      js4.addChangeListener(new SliderChangeListener4());
      js1.addChangeListener(new SliderChangeListener3());
      js2.addChangeListener(new SliderChangeListener2());
      js3.addChangeListener(new SliderChangeListener1());

      // Create an animation thread.
      final FPSAnimator animator = new FPSAnimator(glDrawable, 30, true);
      animator.start();
   }


   @Override
   public void init(GLAutoDrawable drawable)
   {
    //drawable.setGL(new com.jogamp.opengl.TraceGL2((GL2)drawable.getGL(), System.err));

      final GL2 gl = drawable.getGL().getGL2();
      final GLU glu = new GLU();  // get GL Utilities

      // Initialize OpenGL state
      // attributes
      gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f); // white background

      // set up viewing
      gl.glMatrixMode(GL2.GL_PROJECTION);
      gl.glLoadIdentity();
      gl.glOrtho(0.0f, 7.0f, 0.0f, 2.0f, -1.0f, 20f); // orthographic viewing volume
    //glu.gluPerspective(90, 1.0, 0.1, 20); // perspective viewing volume
      gl.glMatrixMode(GL2.GL_MODELVIEW);
   }


   @Override
   public void display(GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glClear(GL2.GL_COLOR_BUFFER_BIT); // clear the graphics panel

      gl.glLoadIdentity();

      gl.glTranslated(d1, 0.0, 0.0);
      gl.glColor3f(0.0f, 1.0f, 0.0f);
      drawBox(width1, height1, drawable);
      gl.glTranslated(0.0, height1, 0.0);

      gl.glPushMatrix();

      gl.glTranslated(d2, 0.0, 0.0);
      gl.glColor3f(1.0f, 1.0f, 0.0f);
      drawBox(width2, height2, drawable);
      gl.glTranslated(0.0, height2, 0.0);
      gl.glTranslated(d3, 0.0, 0.0);
      gl.glColor3f(0.0f, 1.0f, 1.0f);
      drawBox(width3, height3, drawable);

      gl.glPopMatrix();

      gl.glTranslated(d4, 0.0, 0.0);
      gl.glColor3f(1.0f, 1.0f, 0.0f);
      drawBox(width4, height4, drawable);
      gl.glTranslated(0.0, height4, 0.0);
      gl.glTranslated(d5, 0.0, 0.0);
      gl.glColor3f(0.0f, 1.0f, 1.0f);
      drawBox(width5, height5, drawable);

      gl.glFlush(); // flush the buffers
   }


   private static void drawBox(double width, double height, GLAutoDrawable drawable)
   {
      final GL2 gl = drawable.getGL().getGL2();

      gl.glBegin(GL2.GL_TRIANGLES);
         gl.glVertex3d(0.0, 0.0, 0.0);
         gl.glVertex3d(0.0, height, 0.0);
         gl.glVertex3d(width, height, 0.0);

         gl.glVertex3d(width, height, 0.0);
         gl.glVertex3d(width, 0.0, 0.0);
         gl.glVertex3d(0.0, 0.0, 0.0);
      gl.glEnd();
   }


   @Override
   public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h)
   {
   }

   @Override
   public void dispose(GLAutoDrawable drawable)
   {
   }


   public static void main(String[] args)
   {
      javax.swing.SwingUtilities.invokeLater(
         () -> new StackedBoxes2()
      );
   }
}


class SliderChangeListener1 implements ChangeListener
{
   @Override public void stateChanged(ChangeEvent e)
   {
      JSlider source = (JSlider)e.getSource();
      StackedBoxes2.d1 = source.getValue() / 20.0;
   }
}

class SliderChangeListener2 implements ChangeListener
{
   @Override public void stateChanged(ChangeEvent e)
   {
      JSlider source = (JSlider)e.getSource();
      StackedBoxes2.d2 = source.getValue() / 20.0;
   }
}

class SliderChangeListener3 implements ChangeListener
{
   @Override public void stateChanged(ChangeEvent e)
   {
      JSlider source = (JSlider)e.getSource();
      StackedBoxes2.d3 = source.getValue() / 20.0;
   }
}

class SliderChangeListener4 implements ChangeListener
{
   @Override public void stateChanged(ChangeEvent e)
   {
      JSlider source = (JSlider)e.getSource();
      StackedBoxes2.d4 = source.getValue() / 20.0;
   }
}

class SliderChangeListener5 implements ChangeListener
{
   @Override public void stateChanged(ChangeEvent e)
   {
      JSlider source = (JSlider)e.getSource();
      StackedBoxes2.d5 = source.getValue() / 20.0;
   }
}
